﻿using System.Windows;
using System.Windows.Controls;
using System.Collections.ObjectModel;
using System.Collections;
using Telerik.Windows.Controls.DragDrop;
using Telerik.Windows.Controls.TreeView;
using System;
using System.Windows.Media;
using System.Linq;
using Telerik.Windows.Controls;

namespace DragAndDrop_TreeView_ListBox
{
	public partial class MainPage : UserControl
	{
		public MainPage()
		{
			InitializeComponent();

			radTreeView.ItemsSource = CategoryViewModel.Generate();
			IList orderSource = new ObservableCollection<ProductViewModel>();
			foreach (ProductViewModel product in ProductViewModel.Generate(4))
				orderSource.Add(product);
			listBox.ItemsSource = orderSource;

			RadDragAndDropManager.AddDropQueryHandler(listBox, OnDropQuery);
			RadDragAndDropManager.AddDropInfoHandler(listBox, OnDropInfo);
			RadDragAndDropManager.AddDragQueryHandler(listBox, OnDragQuery);
			RadDragAndDropManager.AddDragInfoHandler(listBox, OnDragInfo);
		}

		// OnDragQuery event handler
		private void OnDragQuery(object sender, DragDropQueryEventArgs e)
		{
			System.Windows.Controls.ListBox listBox = sender as System.Windows.Controls.ListBox;
			if (listBox != null)
			{
				IList selectedItems = listBox.SelectedItems.Cast<object>().ToList();
				e.Options.Payload = selectedItems;
			}
			e.QueryResult = true;
			e.Handled = true;
		}

		// OnDropQuery event handler
		private void OnDropQuery(object sender, DragDropQueryEventArgs e)
		{
			ICollection draggedItems = e.Options.Payload as ICollection;
			bool result = draggedItems.Cast<object>().All((object item) => item is ProductViewModel);
			e.QueryResult = result;
			e.Handled = true;
		}

		// OnDragInfo event handler
		private void OnDragInfo(object sender, DragDropEventArgs e)
		{
			System.Windows.Controls.ListBox listBox = sender as System.Windows.Controls.ListBox;
			IEnumerable draggedItems = e.Options.Payload as IEnumerable;
			if (e.Options.Status == DragStatus.DragInProgress)
			{
				//Set up a drag cue:
				TreeViewDragCue cue = new TreeViewDragCue();
				cue.ItemTemplate = listBox.ItemTemplate;
				cue.ItemsSource = draggedItems;
				e.Options.DragCue = cue;
			}
			else if (e.Options.Status == DragStatus.DragComplete)
			{
				IList source = listBox.ItemsSource as IList;
				//foreach (object draggedItem in draggedItems)
				//{
				//    source.Remove(draggedItem);
				//}
			}
		}

		// OnDropInfo event handler
		private void OnDropInfo(object sender, DragDropEventArgs e)
		{
			System.Windows.Controls.ItemsControl wishlist = e.Options.Destination as System.Windows.Controls.ItemsControl;
			ICollection draggedItems = e.Options.Payload as ICollection;
			if (wishlist == null)
				return;
			// Get the drag cu that the TreeView or we have created
			TreeViewDragCue cue = e.Options.DragCue as TreeViewDragCue;
			if (e.Options.Status == DragStatus.DropPossible)
			{
				// Set a suitable text:
				cue.DragActionContent = String.Format("Add {0} item{1} to Wishlist", draggedItems.Count, draggedItems.Count > 1 ? "s" : String.Empty);
				cue.IsDropPossible = true;
				wishlist.Background = this.Resources["DropPossibleBackground"] as Brush;
			}
			else if (e.Options.Status == DragStatus.DropImpossible)
			{
				cue.DragActionContent = null;
				cue.IsDropPossible = false;
			}
			else if (e.Options.Status == DragStatus.DropComplete)
			{
				IList items = wishlist.ItemsSource as IList;
				foreach (object draggedItem in draggedItems)
				{
					items.Add(draggedItem);
				}
			}
			if (e.Options.Status != DragStatus.DropPossible)
			{
				wishlist.Background = new SolidColorBrush(Colors.White);
			}
		}
	}

	public class ExampleTemplateSelector : Telerik.Windows.Controls.DataTemplateSelector
	{
		public override DataTemplate SelectTemplate(object item, DependencyObject container)
		{
			if (item is ProductViewModel)
				return ProductTemplate;
			else if (item is CategoryViewModel)
				return CategoryTemplate;
			return null;
		}

		public DataTemplate ProductTemplate
		{
			get;
			set;
		}

		public DataTemplate CategoryTemplate
		{
			get;
			set;
		}
	}
}
